@@ -3,20 +3,22 @@ use strict;
 use warnings;
 package Email::MIME;
 # ABSTRACT: easy MIME message handling
-$Email::MIME::VERSION = '1.929';
-use Email::Simple 2.102; # crlf handling
+$Email::MIME::VERSION = '1.946';
+use Email::Simple 2.212; # nth header value
 use parent qw(Email::Simple);
 
 use Carp ();
 use Email::MessageID;
 use Email::MIME::Creator;
-use Email::MIME::ContentType 1.016; # type/subtype, not discrete/composite
+use Email::MIME::ContentType 1.022; # parse_content_disposition
 use Email::MIME::Encode;
 use Email::MIME::Encodings 1.314;
 use Email::MIME::Header;
 use Email::MIME::Modifier;
 use Encode 1.9801 ();
-use Scalar::Util qw(reftype);
+use Scalar::Util qw(reftype weaken);
+
+our @CARP_NOT = qw(Email::MIME::ContentType);
 
 #pod =head1 SYNOPSIS
 #pod
@@ -61,7 +63,11 @@ use Scalar::Util qw(reftype);
 #pod   );
 #pod
 #pod   my $email = Email::MIME->create(
-#pod       header_str => [ From => 'casey@geeknest.com' ],
+#pod       header_str => [
+#pod           From => 'casey@geeknest.com',
+#pod           To => [ 'user1@host.com', 'Name <user2@host.com>' ],
+#pod           Cc => Email::Address::XS->new("Display Name \N{U+1F600}", 'user@example.com'),
+#pod       ],
 #pod       parts      => [ @parts ],
 #pod   );
 #pod
@@ -72,7 +78,7 @@ use Scalar::Util qw(reftype);
 #pod         Email::MIME->create( parts => [ @parts ] ),
 #pod       ],
 #pod   );
-#pod   
+#pod
 #pod   # standard modifications
 #pod   $email->header_str_set( 'X-PoweredBy' => 'RT v3.0'      );
 #pod   $email->header_str_set( To            => rcpts()        );
@@ -81,9 +87,9 @@ use Scalar::Util qw(reftype);
 #pod
 #pod   # more advanced
 #pod   $_->encoding_set( 'base64' ) for $email->parts;
-#pod   
+#pod
 #pod   # Quick multipart creation
-#pod   my $quicky = Email::MIME->create(
+#pod   my $email = Email::MIME->create(
 #pod       header_str => [
 #pod           From => 'my@address',
 #pod           To   => 'your@address',
@@ -94,9 +100,9 @@ use Scalar::Util qw(reftype);
 #pod           q[These could be binary too],
 #pod       ],
 #pod   );
-#pod   
+#pod
 #pod   print $email->as_string;
-#pod   
+#pod
 #pod =head1 DESCRIPTION
 #pod
 #pod This is an extension of the L<Email::Simple> module, to handle MIME
@@ -119,7 +125,15 @@ my $NO_ENCODE_RE = qr/
 /ix;
 
 sub new {
-  my $self = shift->SUPER::new(@_);
+  my ($class, $text, $arg, @rest) = @_;
+  $arg ||= {};
+
+  my $encode_check = exists $arg->{encode_check}
+                   ? delete $arg->{encode_check}
+                   : Encode::FB_CROAK;
+
+  my $self = shift->SUPER::new($text, $arg, @rest);
+  $self->encode_check_set($encode_check);
   $self->{ct} = parse_content_type($self->content_type);
   $self->parts;
   return $self;
@@ -141,12 +155,29 @@ sub new {
 #pod
 #pod This method creates a new MIME part. The C<header_str> parameter is a list of
 #pod headers pairs to include in the message. The value for each pair is expected to
-#pod be a text string that will be MIME-encoded as needed.  A similar C<header>
-#pod parameter can be provided in addition to or instead of C<header_str>.  Its
-#pod values will be used verbatim.
+#pod be a text string that will be MIME-encoded as needed.  Alternatively it can be
+#pod an object with C<as_mime_string> method which implements conversion of that
+#pod object to MIME-encoded string.  That object method is called with two named
+#pod input parameters: C<charset> and C<header_name_length>.  It should return
+#pod MIME-encoded representation of the object.  As of 2017-07-25, the
+#pod header-value-as-object code is very young, and may yet change.
+#pod
+#pod In case header name is registered in C<%Email::MIME::Header::header_to_class_map>
+#pod hash then registered class is used for conversion from Unicode string to 8bit
+#pod MIME encoding.  Value can be either string or array reference to strings.
+#pod Object is constructed via method C<from_string> with string value (or values
+#pod in case of array reference) and converted to MIME-encoded string via
+#pod C<as_mime_string> method.
+#pod
+#pod A similar C<header> parameter can be provided in addition to or instead of
+#pod C<header_str>.  Its values will be used verbatim.
 #pod
 #pod C<attributes> is a hash of MIME attributes to assign to the part, and may
-#pod override portions of the header set in the C<header> parameter.
+#pod override portions of the header set in the C<header> parameter. The hash keys
+#pod correspond directly to methods or modifying a message from
+#pod C<Email::MIME::Modifier>. The allowed keys are: content_type, charset, name,
+#pod format, boundary, encoding, disposition, and filename. They will be mapped to
+#pod C<"$attr\_set"> for message modification.
 #pod
 #pod The C<parts> parameter is a list reference containing C<Email::MIME>
 #pod objects. Elements of the C<parts> list can also be a non-reference
@@ -161,12 +192,6 @@ sub new {
 #pod character string to be used as the body.  If you provide a C<body_str>
 #pod parameter, you B<must> provide C<charset> and C<encoding> attributes.
 #pod
-#pod Back to C<attributes>. The hash keys correspond directly to methods or
-#pod modifying a message from C<Email::MIME::Modifier>. The allowed keys are:
-#pod content_type, charset, name, format, boundary, encoding, disposition,
-#pod and filename. They will be mapped to C<"$attr\_set"> for message
-#pod modification.
-#pod
 #pod =cut
 
 my %CT_SETTER = map {; $_ => 1 } qw(
@@ -222,7 +247,13 @@ sub create {
     $CREATOR->_add_to_header(\$header, 'Content-Type' => 'text/plain',);
   }
 
-  my $email = $class->new($header);
+  my %pass_on;
+
+  if (exists $args{encode_check}) {
+    $pass_on{encode_check} = $args{encode_check};
+  }
+
+  my $email = $class->new($header, \%pass_on);
 
   for my $key (keys %attrs) {
     $email->content_type_attribute_set($key => $attrs{$key});
@@ -247,7 +278,7 @@ sub create {
     Carp::confess("body_str was given, but no encoding is defined")
       unless $attrs{encoding};
 
-    my $body_octets = Encode::encode($attrs{charset}, $args{body_str}, 1);
+    my $body_octets = Encode::encode($attrs{charset}, $args{body_str}, $email->encode_check);
     $email->body_set($body_octets);
   }
 
@@ -342,7 +373,7 @@ sub body_str {
     Carp::confess("can't get body as a string for " . $self->content_type);
   }
 
-  my $str = Encode::decode($encoding, $self->body, 1);
+  my $str = Encode::decode($encoding, $self->body, $self->encode_check);
   return $str;
 }
 
@@ -378,8 +409,8 @@ sub parts_multipart {
   my @parts;
   for my $bit (@bits) {
     $bit =~ s/\A[\n\r]+//smg;
-    $bit =~ s/(?<!\x0d)$self->{mycrlf}\Z//smg;
-    my $email = (ref $self)->new($bit);
+    $bit =~ s/(?<!\x0d)$self->{mycrlf}\Z//sm;
+    my $email = (ref $self)->new($bit, { encode_check => $self->encode_check });
     push @parts, $email;
   }
 
@@ -397,8 +428,8 @@ sub debug_structure {
   $level ||= 0;
   my $rv = " " x (5 * $level);
   $rv .= "+ " . ($self->content_type || '') . "\n";
-  my @parts = $self->parts;
-  if (@parts > 1) { $rv .= $_->debug_structure($level + 1) for @parts; }
+  my @parts = $self->subparts;
+  $rv .= $_->debug_structure($level + 1) for @parts;
   return $rv;
 }
 
@@ -409,10 +440,7 @@ sub filename {
   return $gcache{$self} if exists $gcache{$self};
 
   my $dis = $self->header("Content-Disposition") || '';
-  my $attrs
-    = $dis =~ s/^.*?;//
-    ? Email::MIME::ContentType::_parse_attributes($dis)
-    : {};
+  my $attrs = parse_content_disposition($dis)->{attributes};
   my $name = $attrs->{filename}
     || $self->{ct}{attributes}{name};
   return $name if $name or !$force;
@@ -433,6 +461,11 @@ sub invent_filename {
 
 sub default_header_class { 'Email::MIME::Header' }
 
+sub header_str {
+  my $self = shift;
+  $self->header_obj->header_str(@_);
+}
+
 sub header_str_set {
   my $self = shift;
   $self->header_obj->header_str_set(@_);
@@ -443,6 +476,11 @@ sub header_str_pairs {
   $self->header_obj->header_str_pairs(@_);
 }
 
+sub header_as_obj {
+  my $self = shift;
+  $self->header_obj->header_as_obj(@_);
+}
+
 #pod =method content_type_set
 #pod
 #pod   $email->content_type_set( 'text/html' );
@@ -537,6 +575,42 @@ sub content_type_attribute_set {
   $self->_compose_content_type($ct_header);
 }
 
+#pod =method encode_check
+#pod
+#pod =method encode_check_set
+#pod
+#pod   $email->encode_check;
+#pod   $email->encode_check_set(0);
+#pod   $email->encode_check_set(Encode::FB_DEFAULT);
+#pod
+#pod Gets/sets the current C<encode_check> setting (default: I<FB_CROAK>).
+#pod This is the parameter passed to L<Encode/"decode"> and L<Encode/"encode">
+#pod when C<body_str()>, C<body_str_set()>, and C<create()> are called.
+#pod
+#pod With the default setting, Email::MIME may crash if the claimed charset
+#pod of a body does not match its contents (for example - utf8 data in a
+#pod text/plain; charset=us-ascii message).
+#pod
+#pod With an C<encode_check> of 0, the unrecognized bytes will instead be
+#pod replaced with the C<REPLACEMENT CHARACTER> (U+0FFFD), and may end up
+#pod as either that or question marks (?).
+#pod
+#pod See L<Encode/"Handling Malformed Data"> for more information.
+#pod
+#pod =cut
+
+sub encode_check {
+  my ($self) = @_;
+
+  return $self->{encode_check};
+}
+
+sub encode_check_set {
+  my ($self, $val) = @_;
+
+  return $self->{encode_check} = $val;
+}
+
 #pod =method encoding_set
 #pod
 #pod   $email->encoding_set( 'base64' );
@@ -554,7 +628,7 @@ sub encoding_set {
   my ($self, $enc) = @_;
   $enc ||= '7bit';
   my $body = $self->body;
-  $self->header_set('Content-Transfer-Encoding' => $enc);
+  $self->header_raw_set('Content-Transfer-Encoding' => $enc);
   $self->body_set($body);
 }
 
@@ -618,7 +692,7 @@ sub body_str_set {
   Carp::confess("body_str was given, but no charset is defined")
     unless my $charset = $ct->{attributes}{charset};
 
-  my $body_octets = Encode::encode($charset, $body_str, 1);
+  my $body_octets = Encode::encode($charset, $body_str, $self->encode_check);
   $self->body_set($body_octets);
 }
 
@@ -638,7 +712,7 @@ sub disposition_set {
   $dis_header
     ? ($dis_header =~ s/^([^;]+)/$dis/)
     : ($dis_header = $dis);
-  $self->header_set('Content-Disposition' => $dis_header);
+  $self->header_raw_set('Content-Disposition' => $dis_header);
 }
 
 #pod =method filename_set
@@ -655,9 +729,9 @@ sub filename_set {
   my $dis_header = $self->header('Content-Disposition');
   my ($disposition, $attrs);
   if ($dis_header) {
-    ($disposition) = ($dis_header =~ /^([^;]+)/);
-    $dis_header =~ s/^$disposition(?:;\s*)?//;
-    $attrs = Email::MIME::ContentType::_parse_attributes($dis_header) || {};
+    my $struct = parse_content_disposition($dis_header);
+    $disposition = $struct->{type};
+    $attrs = $struct->{attributes};
   }
   $filename ? $attrs->{filename} = $filename : delete $attrs->{filename};
   $disposition ||= 'inline';
@@ -667,7 +741,7 @@ sub filename_set {
     $dis .= qq[; $attr="$val"];
   }
 
-  $self->header_set('Content-Disposition' => $dis);
+  $self->header_raw_set('Content-Disposition' => $dis);
 }
 
 #pod =method parts_set
@@ -697,15 +771,24 @@ sub parts_set {
       $body .= $part->as_string;
     }
     $body .= "$self->{mycrlf}--$bound--$self->{mycrlf}";
-    @{$ct_header}{qw[type subtype]} = qw[multipart mixed]
-      unless grep { $ct_header->{type} eq $_ } qw[multipart message];
+
+    unless (grep { $ct_header->{type} eq $_ } qw[multipart message]) {
+      if (scalar $self->header('Content-Type')) {
+        Carp::carp("replacing non-multipart type ($ct_header->{type}/$ct_header->{subtype}) with multipart/mixed");
+      }
+      @{$ct_header}{qw[type subtype]} = qw[multipart mixed];
+    }
+
     $self->encoding_set('7bit');
     delete $ct_header->{attributes}{charset};
   } elsif (@$parts == 1) {  # setup singlepart
     $body .= $parts->[0]->body;
-    @{$ct_header}{qw[type subtype]}
-      = @{ parse_content_type($parts->[0]->header('Content-Type')) }
-      {qw[type subtype]};
+
+    my $from_ct = parse_content_type($parts->[0]->header('Content-Type'));
+    @{$ct_header}{qw[type subtype]} = @{ $from_ct }{qw[type subtype]};
+
+    $ct_header->{attributes}{charset} = $from_ct->{attributes}{charset};
+
     $self->encoding_set($parts->[0]->header('Content-Transfer-Encoding'));
     delete $ct_header->{attributes}->{boundary};
   }
@@ -736,7 +819,7 @@ sub parts_add {
 #pod   $email->walk_parts(sub {
 #pod       my ($part) = @_;
 #pod       return if $part->subparts; # multipart
-#pod       
+#pod
 #pod       if ( $part->content_type =~ m[text/html]i ) {
 #pod           my $body = $part->body;
 #pod           $body =~ s/<link [^>]+>//; # simple filter example
@@ -756,8 +839,8 @@ sub walk_parts {
 
   my %changed;
 
-  my $walk;
-  $walk = sub {
+  my $walk_weak;
+  my $walk = sub {
     my ($part) = @_;
     $callback->($part);
 
@@ -767,7 +850,7 @@ sub walk_parts {
 
       for my $part (@orig_subparts) {
         my $str = $part->as_string;
-        next unless my $new = $walk->($part);
+        next unless my $new = $walk_weak->($part);
         $differ = 1 if $str ne $new->as_string;
         push @subparts, $new;
       }
@@ -786,6 +869,9 @@ sub walk_parts {
     return $part;
   };
 
+  $walk_weak = $walk;
+  weaken $walk_weak;
+
   my $rv = $walk->($self);
 
   undef $walk;
@@ -797,9 +883,10 @@ sub _compose_content_type {
   my ($self, $ct_header) = @_;
   my $ct = join q{/}, @{$ct_header}{qw[type subtype]};
   for my $attr (sort keys %{ $ct_header->{attributes} }) {
-    $ct .= qq[; $attr="$ct_header->{attributes}{$attr}"];
+    next unless defined (my $value = $ct_header->{attributes}{$attr});
+    $ct .= qq[; $attr="$value"];
   }
-  $self->header_set('Content-Type' => $ct);
+  $self->header_raw_set('Content-Type' => $ct);
   $self->{ct} = $ct_header;
 }
 
@@ -825,11 +912,11 @@ sub _reset_cids {
       return if keys(%cids) == 1;
 
       my $cid = $self->_get_cid;
-      $_->header_set('Content-ID' => "<$cid>") for $self->parts;
+      $_->header_raw_set('Content-ID' => "<$cid>") for $self->parts;
     } else {
       foreach ($self->parts) {
         my $cid = $self->_get_cid;
-        $_->header_set('Content-ID' => "<$cid>")
+        $_->header_raw_set('Content-ID' => "<$cid>")
           unless $_->header('Content-ID');
       }
     }
@@ -848,7 +935,7 @@ Email::MIME - easy MIME message handling
 
 =head1 VERSION
 
-version 1.929
+version 1.946
 
 =head1 SYNOPSIS
 
@@ -893,7 +980,11 @@ by all means keep reading.
   );
 
   my $email = Email::MIME->create(
-      header_str => [ From => 'casey@geeknest.com' ],
+      header_str => [
+          From => 'casey@geeknest.com',
+          To => [ 'user1@host.com', 'Name <user2@host.com>' ],
+          Cc => Email::Address::XS->new("Display Name \N{U+1F600}", 'user@example.com'),
+      ],
       parts      => [ @parts ],
   );
 
@@ -904,7 +995,7 @@ by all means keep reading.
         Email::MIME->create( parts => [ @parts ] ),
       ],
   );
-  
+
   # standard modifications
   $email->header_str_set( 'X-PoweredBy' => 'RT v3.0'      );
   $email->header_str_set( To            => rcpts()        );
@@ -913,9 +1004,9 @@ by all means keep reading.
 
   # more advanced
   $_->encoding_set( 'base64' ) for $email->parts;
-  
+
   # Quick multipart creation
-  my $quicky = Email::MIME->create(
+  my $email = Email::MIME->create(
       header_str => [
           From => 'my@address',
           To   => 'your@address',
@@ -926,7 +1017,7 @@ by all means keep reading.
           q[These could be binary too],
       ],
   );
-  
+
   print $email->as_string;
 
 =head1 DESCRIPTION
@@ -957,12 +1048,29 @@ very long. Added to that, you have:
 
 This method creates a new MIME part. The C<header_str> parameter is a list of
 headers pairs to include in the message. The value for each pair is expected to
-be a text string that will be MIME-encoded as needed.  A similar C<header>
-parameter can be provided in addition to or instead of C<header_str>.  Its
-values will be used verbatim.
+be a text string that will be MIME-encoded as needed.  Alternatively it can be
+an object with C<as_mime_string> method which implements conversion of that
+object to MIME-encoded string.  That object method is called with two named
+input parameters: C<charset> and C<header_name_length>.  It should return
+MIME-encoded representation of the object.  As of 2017-07-25, the
+header-value-as-object code is very young, and may yet change.
+
+In case header name is registered in C<%Email::MIME::Header::header_to_class_map>
+hash then registered class is used for conversion from Unicode string to 8bit
+MIME encoding.  Value can be either string or array reference to strings.
+Object is constructed via method C<from_string> with string value (or values
+in case of array reference) and converted to MIME-encoded string via
+C<as_mime_string> method.
+
+A similar C<header> parameter can be provided in addition to or instead of
+C<header_str>.  Its values will be used verbatim.
 
 C<attributes> is a hash of MIME attributes to assign to the part, and may
-override portions of the header set in the C<header> parameter.
+override portions of the header set in the C<header> parameter. The hash keys
+correspond directly to methods or modifying a message from
+C<Email::MIME::Modifier>. The allowed keys are: content_type, charset, name,
+format, boundary, encoding, disposition, and filename. They will be mapped to
+C<"$attr\_set"> for message modification.
 
 The C<parts> parameter is a list reference containing C<Email::MIME>
 objects. Elements of the C<parts> list can also be a non-reference
@@ -977,12 +1085,6 @@ If C<body_str> is given instead of C<body> or C<parts>, it is assumed to be a
 character string to be used as the body.  If you provide a C<body_str>
 parameter, you B<must> provide C<charset> and C<encoding> attributes.
 
-Back to C<attributes>. The hash keys correspond directly to methods or
-modifying a message from C<Email::MIME::Modifier>. The allowed keys are:
-content_type, charset, name, format, boundary, encoding, disposition,
-and filename. They will be mapped to C<"$attr\_set"> for message
-modification.
-
 =head2 content_type_set
 
   $email->content_type_set( 'text/html' );
@@ -1007,6 +1109,28 @@ These four methods modify common C<Content-Type> attributes. If set to
 C<undef>, the attribute is removed. All other C<Content-Type> header
 information is preserved when modifying an attribute.
 
+=head2 encode_check
+
+=head2 encode_check_set
+
+  $email->encode_check;
+  $email->encode_check_set(0);
+  $email->encode_check_set(Encode::FB_DEFAULT);
+
+Gets/sets the current C<encode_check> setting (default: I<FB_CROAK>).
+This is the parameter passed to L<Encode/"decode"> and L<Encode/"encode">
+when C<body_str()>, C<body_str_set()>, and C<create()> are called.
+
+With the default setting, Email::MIME may crash if the claimed charset
+of a body does not match its contents (for example - utf8 data in a
+text/plain; charset=us-ascii message).
+
+With an C<encode_check> of 0, the unrecognized bytes will instead be
+replaced with the C<REPLACEMENT CHARACTER> (U+0FFFD), and may end up
+as either that or question marks (?).
+
+See L<Encode/"Handling Malformed Data"> for more information.
+
 =head2 encoding_set
 
   $email->encoding_set( 'base64' );
@@ -1076,7 +1200,7 @@ of additional parts.
   $email->walk_parts(sub {
       my ($part) = @_;
       return if $part->subparts; # multipart
-      
+
       if ( $part->content_type =~ m[text/html]i ) {
           my $body = $part->body;
           $body =~ s/<link [^>]+>//; # simple filter example
@@ -1089,20 +1213,50 @@ each. Accepts a code reference as its only argument. The code reference
 will be passed a single argument, the current MIME part within the
 top-level MIME object. All changes will be applied in place.
 
+=head2 header
+
+B<Achtung!>  Beware this method!  In Email::MIME, it means the same as
+C<header_str>, but on an Email::Simple object, it means C<header_raw>.  Unless
+you always know what kind of object you have, you could get one of two
+significantly different behaviors.
+
+Try to use either C<header_str> or C<header_raw> as appropriate.
+
 =head2 header_str_set
 
   $email->header_str_set($header_name => @value_strings);
 
-This behaves like C<header_set>, but expects Unicode (character) strings as the
-values to set, rather than pre-encoded byte strings.  It will encode them as
-MIME encoded-words if they contain any control or 8-bit characters.
+This behaves like C<header_raw_set>, but expects Unicode (character) strings as
+the values to set, rather than pre-encoded byte strings.  It will encode them
+as MIME encoded-words if they contain any control or 8-bit characters.
+
+Alternatively, values can be objects with C<as_mime_string> method.  Same as in
+method C<create>.
 
 =head2 header_str_pairs
 
   my @pairs = $email->header_str_pairs;
 
-This method behaves like C<header_pairs>, returning a list of field name/value
-pairs, but the values have been decoded to character strings, when possible.
+This method behaves like C<header_raw_pairs>, returning a list of field
+name/value pairs, but the values have been decoded to character strings, when
+possible.
+
+=head2 header_as_obj
+
+  my $first_obj = $email->header_as_obj($field);
+  my $nth_obj   = $email->header_as_obj($field, $index);
+  my @all_objs  = $email->header_as_obj($field);
+
+  my $nth_obj_of_class  = $email->header_as_obj($field, $index, $class);
+  my @all_objs_of_class = $email->header_as_obj($field, undef, $class);
+
+This method returns an object representation of the header value.  It instances
+new object via method C<from_mime_string> of specified class.  Input argument
+for that class method is list of the raw MIME-encoded values.  If class argument
+is not specified then class name is taken from the hash
+C<%Email::MIME::Header::header_to_class_map> via key field.  Use class method
+C<< Email::MIME::Header->set_class_for_header($class, $field) >> for adding new
+mapping.
 
 =head2 parts
 
@@ -1212,6 +1366,78 @@ Simon Cozens <simon@cpan.org>
 
 =back
 
+=head1 CONTRIBUTORS
+
+=for stopwords Alex Vandiver Arthur Axel 'fREW' Schmidt Brian Cassidy Dan Book David Steinbrunner Dotan Dimet Geraint Edwards Jesse Luehrs Kurt Anderson Lance A. Brown Matthew Horsfall (alh) memememomo Michael McClimon Pali Shawn Sorichetti Tomohiro Hosaka
+
+=over 4
+
+=item *
+
+Alex Vandiver <alexmv@mit.edu>
+
+=item *
+
+Arthur Axel 'fREW' Schmidt <frioux@gmail.com>
+
+=item *
+
+Brian Cassidy <bricas@cpan.org>
+
+=item *
+
+Dan Book <grinnz@gmail.com>
+
+=item *
+
+David Steinbrunner <dsteinbrunner@pobox.com>
+
+=item *
+
+Dotan Dimet <dotan@corky.net>
+
+=item *
+
+Geraint Edwards <gedge-oss@yadn.org>
+
+=item *
+
+Jesse Luehrs <doy@tozt.net>
+
+=item *
+
+Kurt Anderson <kboth@drkurt.com>
+
+=item *
+
+Lance A. Brown <lance@bearcircle.net>
+
+=item *
+
+Matthew Horsfall (alh) <wolfsage@gmail.com>
+
+=item *
+
+memememomo <memememomo@gmail.com>
+
+=item *
+
+Michael McClimon <michael@mcclimon.org>
+
+=item *
+
+Pali <pali@cpan.org>
+
+=item *
+
+Shawn Sorichetti <ssoriche@coloredblocks.com>
+
+=item *
+
+Tomohiro Hosaka <bokutin@bokut.in>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2004 by Simon Cozens and Casey West.
@@ -1223,21 +1449,50 @@ the same terms as the Perl 5 programming language system itself.
 
 __END__
 
-
+#pod =method header
+#pod
+#pod B<Achtung!>  Beware this method!  In Email::MIME, it means the same as
+#pod C<header_str>, but on an Email::Simple object, it means C<header_raw>.  Unless
+#pod you always know what kind of object you have, you could get one of two
+#pod significantly different behaviors.
+#pod
+#pod Try to use either C<header_str> or C<header_raw> as appropriate.
+#pod
 #pod =method header_str_set
 #pod
 #pod   $email->header_str_set($header_name => @value_strings);
 #pod
-#pod This behaves like C<header_set>, but expects Unicode (character) strings as the
-#pod values to set, rather than pre-encoded byte strings.  It will encode them as
-#pod MIME encoded-words if they contain any control or 8-bit characters.
+#pod This behaves like C<header_raw_set>, but expects Unicode (character) strings as
+#pod the values to set, rather than pre-encoded byte strings.  It will encode them
+#pod as MIME encoded-words if they contain any control or 8-bit characters.
+#pod
+#pod Alternatively, values can be objects with C<as_mime_string> method.  Same as in
+#pod method C<create>.
 #pod
 #pod =method header_str_pairs
 #pod
 #pod   my @pairs = $email->header_str_pairs;
 #pod
-#pod This method behaves like C<header_pairs>, returning a list of field name/value
-#pod pairs, but the values have been decoded to character strings, when possible.
+#pod This method behaves like C<header_raw_pairs>, returning a list of field
+#pod name/value pairs, but the values have been decoded to character strings, when
+#pod possible.
+#pod
+#pod =method header_as_obj
+#pod
+#pod   my $first_obj = $email->header_as_obj($field);
+#pod   my $nth_obj   = $email->header_as_obj($field, $index);
+#pod   my @all_objs  = $email->header_as_obj($field);
+#pod
+#pod   my $nth_obj_of_class  = $email->header_as_obj($field, $index, $class);
+#pod   my @all_objs_of_class = $email->header_as_obj($field, undef, $class);
+#pod
+#pod This method returns an object representation of the header value.  It instances
+#pod new object via method C<from_mime_string> of specified class.  Input argument
+#pod for that class method is list of the raw MIME-encoded values.  If class argument
+#pod is not specified then class name is taken from the hash
+#pod C<%Email::MIME::Header::header_to_class_map> via key field.  Use class method
+#pod C<< Email::MIME::Header->set_class_for_header($class, $field) >> for adding new
+#pod mapping.
 #pod
 #pod =method parts
 #pod
